<?php
/**
 * Blackbird MenuManager Module
 *
 * NOTICE OF LICENSE
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@bird.eu so we can send you a copy immediately.
 *
 * @category            Blackbird
 * @package		Blackbird_MenuManager
 * @copyright           Copyright (c) 2016 Blackbird (http://black.bird.eu)
 * @author		Blackbird Team
 */
namespace FiloBlu\ExtBlackbirdMenuManager\Block\NodeType;

use Magento\Backend\Block\Template\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Profiler;

class Subcategory extends \Blackbird\MenuManager\Block\NodeType\AbstractNodeTypeFront
{
    protected $nodes;

    protected $categoryUrls;
    /**
     * @var ResourceConnection
     */
    private $connection;

    /**
     * @var Profiler
     */
    private $profiler;

    /**
     * @var \Magento\Catalog\Api\CategoryRepositoryInterface
     */
    protected $_categoryRepositoryInterface;

    public function __construct(
        Context $context,
        ResourceConnection $connection,
        Profiler $profiler,
        \Magento\Catalog\Api\CategoryRepositoryInterface $categoryRepositoryInterFace,
        $data = [])
    {
        $this->connection = $connection;
        $this->profiler = $profiler;
        $this->_categoryRepositoryInterface = $categoryRepositoryInterFace;
        parent::__construct($context, $data);
    }


    public function fetchData(array $nodes)
    {
        $this->profiler->start(__METHOD__);
        $localNodes = [];
        $categoryIds = [];
        foreach ($nodes as $node) {
            $localNodes[$node->getId()] = $node;
            $categoryIds[] = (int)$node->getContent();
        }

        $this->nodes = $localNodes;
        $table = $this->connection->getTableName('url_rewrite');
        $select = $this->connection->getConnection('read')
                                   ->select()
                                   ->from($table, ['entity_id', 'request_path'])
                                   ->where('entity_type = ?', 'category')
                                   ->where('store_id = ?', $this->_storeManager->getStore()->getId())
                                   ->where('entity_id IN (' . implode(',', $categoryIds) . ')');
        $this->categoryUrls = $this->connection->getConnection('read')->fetchPairs($select);
        $this->profiler->stop(__METHOD__);
    }


    /**
     * define the url of the node
     *
     * @param $node
     * @return mixed
     */
    public function getUrlNode($node)
    {
        try {
            $category = $this->_categoryRepositoryInterface->get($node->getEntityId());
            $url = $category->getUrl();

            $this->setSubmenu($this->getSubMenuData($category));

        } catch (NoSuchEntityException $e){
            $url = $this->_storeManager->getStore()->getBaseUrl();
        }

        $this->setUrl($url);
        return $url;
    }


    /**
     * @param $nodeId
     * @param $level
     * @param $classes
     * @param $childrenHtml
     * @param $childrenArray
     * @return string
     */
    public function getHtml($nodeId, $level, $classes, $childrenHtml, $childrenArray)
    {
        $node = $this->nodes[$nodeId];
        $node->setUrlPath($this->getUrlNode($node));

        $url = $this->getData('url');
        $classes = $this->getIsActiveClass($url, $node, $classes, $childrenArray);

        return parent::getHtml($nodeId, $level, $classes, $childrenHtml, $childrenArray);
    }

    public function getSubMenuData($category){

        $returnarray = array();

        $childrenCategories = $category->getChildrenCategories();
        foreach($childrenCategories as $subCat){
            $returnarray[] = array(
                'name'  => $subCat->getName(),
                'url'   => $subCat->getUrl()
            );
        }

        return $returnarray;
    }
    
      /**
     * method to prepare the template
     *
     * @param $type
     * @param $menuIdentifier
     * @return $this
     */
    public function prepareTemplate($type, $menuIdentifier)
    {
        //Custom template if existing
        //$menuidentifier is the identifier of the menu used for every pages defined in the default.xml
        $this->setTemplate('FiloBlu_ExtBlackbirdMenuManager::menu/view/' . $menuIdentifier . '/nodetype/'. $type .'.phtml');

        if(!$this->getTemplateFile()) {
            //default case
            $this->setTemplate('FiloBlu_ExtBlackbirdMenuManager::menu/view/default/nodetype/'. $type .'.phtml');
        }

        return $this;
    }
}